<?php
namespace ReynaertBe\DIP;

class DIP {
    private static $email = ""; // WC config setting
    private static $password = ""; // WC config setting
    private static $resourceBase = ""; // WC config setting, wordt iets als https://defactu.re/api/
    private static $resourceToken = "token/";
    public static function getEmail() : string { return self::$email;}
    public static function getPassword() : string { return self::$password;}
    public static function getResourceBase() : string { return self::$resourceBase;}
    public static function getResourceToken() : string { return self::getResourceBase().self::$resourceToken;}


    private $resource = null;
    private $method = 'POST';
    private $parameters = [];
    private $body = null;
    private $service = null;
    private $error = null;

    const METHOD_GET = 'GET';
    const METHOD_POST = 'POST';
    const METHOD_PUT = 'PUT';
    const METHOD_DELETE = 'DEL';
    const METHOD_PATCH = 'PATCH';


    private static function addTrailingSlash($path) {
        return rtrim($path, '/') . '/'; // verwijder en zet terug erbij
    }

    public function setResource($resource,$subdirs=[]) : self {
        $this->resource = $resource;
        if (!is_null($subdirs) && count($subdirs)!=0) {
            $this->resource = self::addTrailingSlash($this->resource);
            foreach ($subdirs as $subdir) $this->resource.= $subdir.'/';
        }
        return $this;
    }

    public function setMethod($method = self::METHOD_GET) {
        $this->method = $method;
        return $this;
    }

    /**
     * Voeg parameters toe als array, json of key value paar
     * Note this will overwrite existing key/value pairs
     */
    public function addParameter($key,$value=null) : self {
        if (is_null($value)) {
            if (is_array($key)) {
                foreach ($key as $k => $v) $this->parameters[$k]=$v;
            }
        } else
            $this->parameters[$key]=$value;
        return $this;
    }
    private function getParameters(){
        return $this->parameters;
    }

    public function getBody() {
        return $this->body;
    }
    public function setBody($val) :self {
        if (is_array($val)) $val = json_decode(json_encode($val));
        else if (is_string($val)) $val = json_decode($val);
        $this->body = $val;
        return $this;
    }



    public function getError() { return $this->error; }
    public function hasError() { return (!is_null($this->error) && $this->error->hasError()); }

    public static function jsonDecode($p) {return json_decode($p);}

    public function call($transform='FactureDIP::jsonDecode', $resource=null, $data=false) {
        if (is_null($resource)) $resource = $this->resource;
        if (!$data) {
            if (!is_null($this->getBody()) && ($this->method==self::METHOD_POST || $this->method==self::METHOD_PATCH))
                $data = json_encode($this->getBody());
            else
                $data = $this->getParameters();
        }
        $result = $this->service->CallAPI($this->method, $resource, $data, $transform);

        //todo Te vervangen door WP_error (ooit)
        $error = null;
        if (is_array($result) && isset($result[0]->_error)) $error = $result[0];
        else if (isset($result->_error)) $error = $result;
        if (!is_null($error)) {
            $this->error->setError($error->_error);
            $this->error->setMajor($error->_major);
            $this->error->setMessage($error->message." (code: ".$error->_error." - ".$resource.")");
        }

        return $result;
    }


    public function __construct() {
        $base = get_option('woocommerce_reynaert_defacture_url');
        if (strlen($base)!=0) {
            $url = parse_url($base);
            self::$resourceBase = self::addTrailingSlash(($url['scheme']??'https')."://". $url['host'] )."api/";
            self::$email = get_option('woocommerce_reynaert_defacture_username');
            self::$password = get_option('woocommerce_reynaert_defacture_password');
        } else {
            throw new \Exception('There is no data to communicate with defactu.re web application. Please enter the settings under WooCommerce > Settings > DeFactu.re');
        }
        $this->service = new Service();
        $this->error = new JSONError();
        return $this;
    }

    public function debug($on=true) : self {
        $this->service->debug($on);
        return $this;
    }

    //public static function boolToBit($b) { return is_null($b)?null:($b?1:0); }
}